<?php /* ***** Orca Search - Searching Engine ***********************


*********************************************************************
***** Output Documentation ******************************************
* Search Query:
*  - Only set if an actual query exists, unset otherwise
* 1) $_QUERY array with hash:
*   -> "original" - String
*        => Original query as typed
*   -> "category" - String
*        => Selected category, empty string if none
*   -> "allterms" - Array
*        => Unfiltered terms in no order (includes ignored terms)
*   -> "terms" - Array
*        => Filtered terms in no order (terms actually searched for)
*   -> "sorted" - String
*        => Filtered terms sorted alphabetically (cache index column)
*   -> "and" - Array
*        => All terms which were marked with +
*   -> "not" - Array
*        => All terms which were marked with - or !
*   -> "or" - Array
*        => All terms which were not marked
*   -> "andor" - Array
*        => Combined "or" and "and" arrays stripped of + marks
*
* Search Info:
* 1) $_SDATA['now'] - Float
*   -> *NIX timestamp with microtime() when script started
*   -> Use array_sum(explode(" ", microtime())) - $_SDATA['now']
*       to find execution-time
* 2) $_SDATA['totalRows'] - Integer
*   -> Number of searchable rows indexed
* 3) $_SDATA['categories'] - Array
*   -> List of all categories available from searchable pages
*
* Results:
* 1) If no database connection:
*   -> $_DDATA['online'] = false
*     -> $_DDATA['error'] = MySQL error message (if exists)
*     -> $_DDATA['errno'] = MySQL error code (if exists)
*
* 2) If the database is locked, and there are no cached results:
*   -> $_DDATA['online'] = true;
*   -> $_RESULTS == NULL
*
* 3) If no results are found:
*   -> $_DDATA['online'] = true
*   -> Empty $_RESULTS array
*
* 4) If results were found:
*   -> $_DDATA['online'] = true
*   -> Relevance-ordered array $_RESULTS with each item having hash:
*     -> "title" - String
*          => Title of match (or URI if no title)
*     -> "description" - String
*          => Meta (or table-assigned) description
*     -> "category" - String
*          => Assigned category, pre-filtered by $_GET['c']
*     -> "uri" - String
*          => Full URI of match
*     -> "matchURI" - String
*          => Full URI with matching terms highlighted
*     -> "filetype" - String
*          => Page filetype: html, txt, pdf, jpg, etc.
*     -> "matchText" - String
*          => Collection of matched text from entry
*     -> "relevance" - Float
*          => Relevance score
****************************************************************** */

/* ******************************************************************
******** Functions *********************************************** */
function OS_mysqlFormat($input, $neg) {
  $input = ltrim($input, "!+-");
  if (strpos($input, " ") !== false) {
    $type = "REGEXP";
    $input = str_replace(" ", '[,.?!]?[ \-][,.?!]?', preg_quote($input));
  } else {
    $type = "LIKE";
    $input = "%$input%";
  }
  return (($neg) ? "NOT " : "")."$type '".addslashes($input)."'";
}

function OS_addRelevance(&$value, $terms, $multiplier) {
  global $_VDATA;

  $areas = array("title", "body", "keywords", "wtags", "uri");
  $order = array_flip($areas);
  $multi = -1;
  $foundlimit = 3;
  $lbody = strtolower($value['body']);

  foreach ($terms as $term) {
    $relevance = $value['relevance'];
    $phrase = false;
    $match = array();
    $found = array();

    if (strpos($term, " ")) {
      $term = str_replace(" ", '[,.?!]?[ \-][,.?!]?', preg_quote($term, "/"));
      $phrase = true;
    } else if ($_VDATA['s.latinacc'] == "true") {
      $term = OS_latinAccents(preg_quote($term, "/"), ($_VDATA['sp.utf8'] == "true") ? true : false);
    } else $term = preg_quote($term, "/");

    foreach ($areas as $area) {
      if ($_VDATA['s.weight'][$order[$area]] > 0) {
        if ($phrase || $_VDATA['s.latinacc'] == "true") {
          preg_match_all("/$term/i", $value[$area], $match[$area]);
          $found[$area] = count($match[$area][0]);
        } else $found[$area] = substr_count(strtolower($value[$area]), strtolower($term));
      } else $found[$area] = 0;
    }

    reset($areas);
    while (list($key, $val) = each($areas))
      $value['relevance'] += $_VDATA['s.weight'][$key] * min($foundlimit, $found[$val]) * $multiplier;
    if ($value['relevance'] > $relevance) $multi++;

    unset($matchtext);
    if ($phrase || $_VDATA['s.latinacc'] == "true") {
      if (isset($match['body'][0][0]) && ($firstpos = strpos($lbody, strtolower($match['body'][0][0]))) !== false)
        $matchtext = substr("  ".$value['body']."  ", max(0, $firstpos - 50), 140 + strlen($match['body'][0][0]));
    } else if ($found['body']) {
      $firstpos = strpos($lbody, strtolower($term));
      $matchtext = substr("  ".$value['body']."  ", max(0, $firstpos - 50), 140 + strlen($term));
    }

    if (isset($matchtext) && ((strlen($value['matchText']) + strlen($matchtext)) < $_VDATA['s.matchingtext']) && !preg_match("/$term/i", $value['matchText'])) {
      if ($_VDATA['sp.utf8'] == "true") {
        $matchtext = preg_replace(array("/^.*?(?=[\xC2-\xDF\xE0-\xF4\s])/s", "/(\xF4[\x80-\x8F]?[\x80-\xBF]?|[\xF1-\xF3][\x80-\xBF]{,2}|\xF0[\x90-\xBF]?[\x80-\xBF]?|\xED[\x80-\x9F]?|[\xE1-\xEC\xEE\xEF][\x80-\xBF]?|\xE0[\xA0-\xBF]?|[\xC2-\xDF]|\w+)$/"), "", $matchtext);
      } else $matchtext = preg_replace(array("/^[^\s]*\s/", "/\s[^\s]*$/"), "", $matchtext);
      $value['matchText'] .= $matchtext." ... ";
    }
  }

  $value['relevance'] *= pow($_VDATA['s.weight'][5], $multi);
}

function OS_latinAccents($_, $utf8 = true) {
  static $accrep = array();

  if ($utf8) {
    $_ = str_replace(array("AE", "Ae", "aE", "ae"), "(ae|[])", $_);
    $_ = str_replace(array("A", "a"), "(a|[])", $_);
    $_ = str_replace(array("C", "c"), "(c|[])", $_);
    $_ = str_replace(array("E", "e"), "(e|[])", $_);
    $_ = str_replace(array("I", "i"), "(i|[])", $_);
    $_ = str_replace(array("N", "n"), "(n|[])", $_);
    $_ = str_replace(array("O", "o"), "(o|[])", $_);
    $_ = str_replace(array("S", "s"), "(s|ß)", $_);
    $_ = str_replace(array("T", "t"), "(t|[])", $_);
    $_ = str_replace(array("U", "u"), "(u|[])", $_);
    $_ = str_replace(array("Y", "y"), "(y|[])", $_);

  } else {
    if (!count($accrep)) {
      $accrep = array(
        "" => '([]|&(\101\105|\141\145)\154\151\147;)',
        "" => '([]|&(\101\105|\141\145)\154\151\147;)',
        "" => '([]|&[\101\141]\147\162\141\166\145;)',
        "" => '([]|&[\101\141]\141\143\165\164\145;)',
        "" => '([]|&[\101\141]\143\151\162\143;)',
        "" => '([]|&[\101\141]\164\151\154\144\145;)',
        "" => '([]|&[\101\141]\165\155\154;)',
        "" => '([]|&[\101\141]\162\151\156\147;)',
        "" => '([]|&[\101\141]\147\162\141\166\145;)',
        "" => '([]|&[\101\141]\141\143\165\164\145;)',
        "" => '([]|&[\101\141]\143\151\162\143;)',
        "" => '([]|&[\101\141]\164\151\154\144\145;)',
        "" => '([]|&[\101\141]\165\155\154;)',
        "" => '([]|&[\101\141]\162\151\156\147;)',
        "" => '([]|&[\103\143]\143\145\144\151\154;)',
        "" => '([]|&[\103\143]\143\145\144\151\154;)',
        "" => '([]|&(\105\124\110|\145\164\150);)',
        "" => '([]|&[\105\145]\147\162\141\166\145;)',
        "" => '([]|&[\105\145]\141\143\165\164\145;)',
        "" => '([]|&[\105\145]\143\151\162\143;)',
        "" => '([]|&[\105\145]\165\155\154;)',
        "" => '([]|&(\105\124\110|\145\164\150);)',
        "" => '([]|&[\105\145]\147\162\141\166\145;)',
        "" => '([]|&[\105\145]\141\143\165\164\145;)',
        "" => '([]|&[\105\145]\143\151\162\143;)',
        "" => '([]|&[\105\145]\165\155\154;)',
        "" => '([]|&[\111\151]\147\162\141\166\145;)',
        "" => '([]|&[\111\151]\141\143\165\164\145;)',
        "" => '([]|&[\111\151]\143\151\162\143;)',
        "" => '([]|&[\111\151]\165\155\154;)',
        "" => '([]|&[\111\151]\147\162\141\166\145;)',
        "" => '([]|&[\111\151]\141\143\165\164\145;)',
        "" => '([]|&[\111\151]\143\151\162\143;)',
        "" => '([]|&[\111\151]\165\155\154;)',
        "" => '([]|&[\116\156]\164\151\154\144\145;)',
        "" => '([]|&[\116\156]\164\151\154\144\145;)',
        "" => '([]|&[\117\157]\147\162\141\166\145;)',
        "" => '([]|&[\117\157]\141\143\165\164\145;)',
        "" => '([]|&[\117\157]\143\151\162\143;)',
        "" => '([]|&[\117\157]\164\151\154\144\145;)',
        "" => '([]|&[\117\157]\165\155\154;)',
        "" => '([]|&[\117\157]\163\154\141\163\150;)',
        "" => '([]|&[\117\157]\147\162\141\166\145;)',
        "" => '([]|&[\117\157]\141\143\165\164\145;)',
        "" => '([]|&[\117\157]\143\151\162\143;)',
        "" => '([]|&[\117\157]\164\151\154\144\145;)',
        "" => '([]|&[\117\157]\165\155\154;)',
        "" => '([]|&[\117\157]\163\154\141\163\150;)',
        "" => '(|&\163\172\154\151\147;)',
        "" => '([]|&(\124\110\117\122\116|\164\150\157\162\156);)',
        "" => '([]|&(\124\110\117\122\116|\164\150\157\162\156);)',
        "" => '([]|&[\125\165]\147\162\141\166\145;)',
        "" => '([]|&[\125\165]\141\143\165\164\145;)',
        "" => '([]|&[\125\165]\143\151\162\143;)',
        "" => '([]|&[\125\165]\165\155\154;)',
        "" => '([]|&[\125\165]\147\162\141\166\145;)',
        "" => '([]|&[\125\165]\141\143\165\164\145;)',
        "" => '([]|&[\125\165]\143\151\162\143;)',
        "" => '([]|&[\125\165]\165\155\154;)',
        "" => '([]|&[\131\171]\141\143\165\164\145;)',
        "" => '([]|&[\131\171]\165\155\154;)',
        "" => '([]|&[\131\171]\141\143\165\164\145;)',
        "" => '([]|&[\131\171]\165\155\154;)'
      );
    }
    $_ = strtr($_, $accrep);
    $_ = str_replace(array("AE", "Ae", "aE", "ae"), "(ae||)", $_);
    $_ = str_replace(array("A", "a"), "([a]|&a(\147\162\141\166\145|\141\143\165\164\145|\143\151\162\143|\164\151\154\144\145|\165\155\154|\162\151\156\147);)", $_);
    $_ = str_replace(array("C", "c"), "([c]|&c\143\145\144\151\154;)", $_);
    $_ = str_replace(array("E", "e"), "([e]|&e((?<=\105)\124\110|(?<=\145)\164\150|\147\162\141\166\145|\141\143\165\164\145|\143\151\162\143|\165\155\154;)", $_);
    $_ = str_replace(array("I", "i"), "([i]|&i(\147\162\141\166\145|\141\143\165\164\145|\143\151\162\143|\165\155\154);)", $_);
    $_ = str_replace(array("N", "n"), "([n]|&n\164\151\154\144\145;)", $_);
    $_ = str_replace(array("O", "o"), "([o]|&o(\147\162\141\166\145|\141\143\165\164\145|\143\151\162\143|\164\151\154\144\145|\165\155\154|\163\154\141\163\150);)", $_);
    $_ = str_replace(array("S", "s"), "([s]|&\163\172\154\151\147;)", $_);
    $_ = str_replace(array("T", "t"), "([t]|&(\124\110\117\122\116|\164\150\157\162\156);)", $_);
    $_ = str_replace(array("U", "u"), "([u]|&u(\147\162\141\166\145|\141\143\165\164\145|\143\151\162\143|\165\155\154);)", $_);
    $_ = str_replace(array("Y", "y"), "([yݟ]|&y(\141\143\165\164\145|\165\155\154);)", $_);
  }
  return $_;
}

function OS_outputFormat($string, $terms) {
  global $_VDATA;

  $string = str_replace(" ... ", " %%%%%%%%%...%%%%/%%%% ", $string);
  foreach ($terms as $term) {
    $term = str_replace(" ", '[,.?!]?[ \-][,.?!]?', preg_quote($term, "/"));
    if ($_VDATA['s.latinacc'] == "true") $term = OS_latinAccents($term, ($_VDATA['sp.utf8'] == "true") ? true : false);
    $string = preg_replace("/($term)/i", "%%%%%%%%%$1%%%%/%%%%", $string);
  }

  $string = strtr($string, array("<" => "&lt;", ">" => "&gt;"));
  $string = str_replace(array("%%%%/%%%%", "%%%%%%%%%", "</strong><strong>"), array("</strong>", "<strong>", ""), $string);
  return $string;
}


/* ******************************************************************
******** Setup *************************************************** */
header("Orcascript: Search_Engine");

if ($_DDATA['online']) {
  $_SDATA['noSearch'] = array_filter(array_map("trim", explode("\n", $_VDATA['s.ignore'])));

  $_SDATA['lq'] = ($_VDATA['s.orphans'] == "show") ? " AND (`status`='OK' OR `status`='Orphan')" : " AND `status`='OK'";

  $_SDATA['nq'] = "";
  foreach ($_SDATA['noSearch'] as $noSearch)
    $_SDATA['nq'] .= " AND `uri` NOT ".(($noSearch{0} == "*") ? "REGEXP '".substr($noSearch, 1)."'": " LIKE '%{$noSearch}%'");

  list($_SDATA['totalRows']) = mysql_fetch_array(mysql_query("SELECT COUNT(*) FROM `{$_DDATA['tablename']}` WHERE `unlist`='false'{$_SDATA['lq']}{$_SDATA['nq']};", $_DDATA['link']));
  $_RESULTS = array();

  $_SDATA['categories'] = array();
  $select = mysql_query("SELECT DISTINCT `category` FROM `{$_DDATA['tablename']}` WHERE `unlist`='false'{$_SDATA['lq']}{$_SDATA['nq']} ORDER BY `category`;", $_DDATA['link']);
  while ($row = mysql_fetch_assoc($select)) $_SDATA['categories'][] = $row['category'];

  $_GET['c'] = (isset($_GET['c'])) ? $_GET['c'] : "";
  if (!in_array($_GET['c'], $_SDATA['categories'])) $_GET['c'] = "";

  if ($_VDATA['s.cachetime'] < (time() - $_VDATA['s.cachereset'] * 86400)) {
    if ($address = trim($_VDATA['s.cacheemail'])) {
      require "phpmailer.php";
      $mail = new PHPMailer();
      $mail->From = $_SERVER['SERVER_ADMIN'];
      $mail->FromName = "Orca Search Spider";
      $mail->CharSet = $_VDATA['c.charset'];

      $address = explode(" ", preg_replace("/[\"<>]/", "", $address));
      while (count($address) > 2) {
        $str = array_shift($address);
        $address[0] = $str + $address[0];
      }
      if (count($address) == 1) array_unshift($address, "");

      $mail->AddAddress($address[1], $address[0]);
      $mail->Subject = "{$_LANG['0lo']}: {$_SERVER['HTTP_HOST']} Orca Search";
      $mail->Body = sprintf($_LANG['0m2'], date("Y-m-d", $_VDATA['s.cachetime']), date("Y-m-d"))."\n\n";
      $mail->Body .= "{$_LANG['0m3']}\n  http://{$_SERVER['HTTP_HOST']}{$_SERVER['REQUEST_URI']}\n\n";
      $mail->Body .= " {$_LANG['0lr']}   {$_LANG['0lp']}\n";
      $mail->Body .= "______ _______________________________________________________________\n";

      $select = mysql_query("SELECT `hits`, `query` FROM `{$_DDATA['tablestat']}` ORDER BY `hits` DESC, `query`;", $_DDATA['link']);
      while ($row = mysql_fetch_assoc($select))
        $mail->Body .= str_pad($row['hits'], 5, " ", STR_PAD_LEFT)."   ".substr($row['query'], 0, 60)."\n";

      $mail->Body .= "\n\n";
      $mail->Body .= "______________________________________________________________________\n";
      $mail->Body .= "Orca Search {$_SDATA['version']}";

      $mail->Send();

    }
    $truncate = mysql_query("TRUNCATE TABLE `{$_DDATA['tablestat']}`;", $_DDATA['link']);
    OS_setData("s.cachetime", time());

  } else {
    list($show) = mysql_fetch_array(mysql_query("SELECT SUM(LENGTH(`cache`)) FROM `{$_DDATA['tablestat']}`;", $_DDATA['link']));
    while ($show > $_VDATA['s.cachelimit'] * 1024) {
      $row = mysql_fetch_assoc(mysql_query("SELECT `query` FROM `{$_DDATA['tablestat']}` WHERE `cache`!='' ORDER BY `lasthit` LIMIT 1;", $_DDATA['link']));
      $update = mysql_query("UPDATE `{$_DDATA['tablestat']}` SET `cache`='' WHERE `query`='".addslashes($row['query'])."';", $_DDATA['link']);
      if (mysql_affected_rows() <= 0) break;
      list($show) = mysql_fetch_array(mysql_query("SELECT SUM(LENGTH(`cache`)) FROM `{$_DDATA['tablestat']}`;", $_DDATA['link']));
    }
    $optimize = mysql_query("OPTIMIZE TABLE `{$_DDATA['tablestat']}`;", $_DDATA['link']);
  }

  /* ****************************************************************
  ******** Search ************************************************ */
  if (isset($_GET['q']) && $_GET['q'] = trim($_GET['q'])) {
    $_QUERY = array();

    $_QUERY['query'] = $_QUERY['original'] = $_GET['q'];
  
    preg_match_all("/[!+\-]?\".*?\"/", $_QUERY['query'], $quotes);
    $_QUERY['terms'] = str_replace('"', "", $quotes[0]);
    $_QUERY['query'] = preg_replace(array("/[!+\-]?\".*?\"/", "/\"/", "/\s{2,}/"), array("", "", " "), $_QUERY['query']);
    $_QUERY['terms'] = array_merge($_QUERY['terms'], explode(" ", $_QUERY['query']));
    $_QUERY['allterms'] = $_QUERY['terms'];
    $_QUERY['terms'] = array_filter($_QUERY['terms'], create_function('$value', 'return (strlen($value) >= '.$_VDATA['s.termlength'].') ? true : false;'));
    $_QUERY['terms'] = array_slice($_QUERY['terms'], 0, $_VDATA['s.termlimit']);

    $_QUERY['sorted'] = $_QUERY['terms'];
    sort($_QUERY['sorted']);
    $_QUERY['sorted'] = addslashes(stripslashes(implode(" ", $_QUERY['sorted'])));
    list($_SDATA['found']) = mysql_fetch_array(mysql_query("SELECT COUNT(*) FROM `{$_DDATA['tablestat']}` WHERE `query`='{$_QUERY['sorted']}';", $_DDATA['link']));
    $_SDATA['count'] = mysql_query("SELECT `cache` FROM `{$_DDATA['tablestat']}` WHERE `query`='{$_QUERY['sorted']}' AND LENGTH(`cache`)>5;", $_DDATA['link']);

    if (count($_QUERY['terms'])) {
      if (!mysql_num_rows($_SDATA['count'])) {
        if ($_VDATA['sp.lock'] == "false" || $_VDATA['sp.seamless'] == "true") {
          $_QUERY['and'] = preg_grep("/^\+/", $_QUERY['terms']);
          $_QUERY['not'] = preg_grep("/^[!\-]/", $_QUERY['terms']);
          $_QUERY['or'] = array_diff($_QUERY['terms'], $_QUERY['and'], $_QUERY['not']);
          array_walk($_QUERY['and'], create_function('&$v, $k', '$v = substr($v, 1);'));
          array_walk($_QUERY['not'], create_function('&$v, $k', '$v = substr($v, 1);'));
          $_QUERY['andor'] = array_merge($_QUERY['and'], $_QUERY['or']);

          $_QUERY['typey'] = preg_grep("/^filetype:\w+/", $_QUERY['andor']);
          $_QUERY['andor'] = array_diff($_QUERY['andor'], $_QUERY['typey']);
          $_QUERY['and'] = array_diff($_QUERY['and'], $_QUERY['typey']);
          $_QUERY['or'] = array_diff($_QUERY['or'], $_QUERY['typey']);
          $_QUERY['typen'] = preg_grep("/^filetype:\w+/", $_QUERY['not']);
          $_QUERY['not'] = array_diff($_QUERY['not'], $_QUERY['typen']);

          $mq = "";
          foreach ($_QUERY['not'] as $not) {
            $shot = OS_mysqlFormat($not, true);
            $mq .= " AND `body` $shot AND `title` $shot AND `keywords` $shot";
          }
          foreach ($_QUERY['and'] as $and) {
            $shot = OS_mysqlFormat($and, false);
            $mq .= " AND (`body` $shot OR `title` $shot OR `keywords` $shot)";
          }
          foreach ($_QUERY['typey'] as $types) {
            $types = trim(preg_replace("/^filetype:/", "", $types));
            if (preg_match("/\w+/", $types)) $mq .= " AND `ctype`='".(($types = $_MIME->get_ctype($types)) ? $types : "none")."'";
          }
          foreach ($_QUERY['typen'] as $types) {
            $types = trim(preg_replace("/^filetype:/", "", $types));
            if (preg_match("/\w+/", $types)) $mq .= " AND `ctype`!='".(($types = $_MIME->get_ctype($types)) ? $types : "none")."'";
          }

          $select = mysql_unbuffered_query("SELECT * FROM `{$_DDATA['tablename']}` WHERE `unlist`='false'{$_SDATA['lq']}{$_SDATA['nq']}{$mq};");

          while ($uri = mysql_fetch_assoc($select)) {
            $uri['relevance'] = 0;
            $uri['bonus'] = 1;
            $uri['matchText'] = "";
            OS_addRelevance($uri, $_QUERY['and'], $_VDATA['s.weight'][6]);
            OS_addRelevance($uri, $_QUERY['or'], 1);

            if ($uri['matchText']) {
              $uri['matchText'] = trim(substr($uri['matchText'], 0, strlen($uri['matchText']) - 5));
              if (preg_match("/^[^A-Z]/", $uri['matchText']{0})) $uri['matchText'] = " ... ".$uri['matchText'];
              if (preg_match("/[^.?!]/", $uri['matchText']{strlen($uri['matchText']) - 1})) $uri['matchText'] .= " ... ";
              $uri['matchText'] = str_replace(array("\n", "\r"), "", $uri['matchText']);
            } else if (trim($uri['description'])) {
              $uri['matchText'] = $uri['description'];
            } else if (trim($uri['body'])) {
              $uri['matchText'] = substr($uri['body'], mt_rand(0, strlen($uri['body']) - (int)($_VDATA['s.matchingtext'] / 3)), (int)($_VDATA['s.matchingtext'] / 3));
              if ($_VDATA['sp.utf8'] == "true") {
                $uri['matchText'] = preg_replace(array("/^.*?(?=[\xC2-\xDF\xE0-\xF4\s])/s", "/(\xF4[\x80-\x8F]?[\x80-\xBF]?|[\xF1-\xF3][\x80-\xBF]{,2}|\xF0[\x90-\xBF]?[\x80-\xBF]?|\xED[\x80-\x9F]?|[\xE1-\xEC\xEE\xEF][\x80-\xBF]?|\xE0[\xA0-\xBF]?|[\xC2-\xDF]|\w+)$/"), "", $uri['matchText']);
              } else $uri['matchText'] = preg_replace(array("/^[^\s]*\s/", "/\s[^\s]*$/"), "", $uri['matchText']);
              $uri['matchText'] .= " ... ";
            }

            $uri['matchText'] = OS_outputFormat($uri['matchText'], $_QUERY['andor']);
            $uri['title'] = OS_outputFormat($uri['title'], $_QUERY['andor']);
            $uri['matchURI'] = ((float)$_VDATA['s.weight'][4] > 0) ? OS_outputFormat($uri['uri'], $_QUERY['andor']) : $uri['uri'];
            $uri['description'] = OS_outputFormat($uri['description'], array());

            if ($uri['relevance'])
              $_RESULTS[] = array(
                "title"       => $uri['title'],
                "description" => $uri['description'],
                "category"    => $uri['category'],
                "uri"         => $uri['uri'],
                "matchURI"    => $uri['matchURI'],
                "filetype"    => $uri['ctype'],
                "matchText"   => $uri['matchText'],
                "relevance"   => $uri['relevance']
              );
          }

          if (count($_RESULTS)) {
            usort($_RESULTS, create_function('$a, $b', 'return($a["relevance"]==$b["relevance"])?0:(($a["relevance"]>$b["relevance"])?-1:1);'));
            $_RESULTS = array_slice($_RESULTS, 0, ($_VDATA['s.resultlimit']) ? $_VDATA['s.resultlimit'] : max(5, min(100, ceil($_SDATA['totalRows'] / 6))));
          }

          if ($_VDATA['s.cachelimit']) {
            $rStore = serialize($_RESULTS);
            $rStore = ($_VDATA['s.cachegzip'] == "on") ? mysql_real_escape_string(gzcompress($rStore)) : addslashes(stripslashes($rStore));
          } else $rStore = "";

          if (!$_SDATA['found']) {
            $insert = mysql_query("INSERT INTO `{$_DDATA['tablestat']}` SET
              `query`='{$_QUERY['sorted']}',
              `results`=".count($_RESULTS).",
              `astyped`='".addslashes($_QUERY['original'])."',
              `lasthit`=UNIX_TIMESTAMP(),
              `cache`='{$rStore}'
            ;", $_DDATA['link']);
          } else {
            $update = mysql_query("UPDATE `{$_DDATA['tablestat']}` SET
              `results`=".count($_RESULTS).",
              `hits`=`hits`+1,
              `lasthit`=UNIX_TIMESTAMP(),
              `cache`='{$rStore}',
              `astyped`='".addslashes($_QUERY['original'])."'
            WHERE `query`='{$_QUERY['sorted']}';", $_DDATA['link']);
          }

        } else {
          $_RESULTS = NULL;
          if ($_VDATA['sp.progress'] < time() - 60) OS_setData("sp.lock", "false");
        }

      } else {
        $_RESULTS = mysql_result($_SDATA['count'], 0, "cache");
        $_RESULTS = unserialize(($_VDATA['s.cachegzip'] == "on") ? gzuncompress($_RESULTS) : stripslashes($_RESULTS));

        if (!isset($_GET['start'])) {
          $update = mysql_query("UPDATE `{$_DDATA['tablestat']}` SET
            `hits`=`hits`+1,
            `lasthit`=UNIX_TIMESTAMP(),
            `astyped`='".addslashes($_QUERY['original'])."'
          WHERE `query`='{$_QUERY['sorted']}';", $_DDATA['link']);
        }
      }

      $_QUERY['category'] = $_GET['c'];
      if ($_QUERY['category'] != "") {
        $_RESULTS = array_filter($_RESULTS, create_function('$v', 'global $_QUERY; return ($v[\'category\'] != $_QUERY[\'category\']) ? false : true;'));
        $_RESULTS = array_values($_RESULTS);
      }
    }
  }

  /* ****************************************************************
  ******** Trigger Another Spider ******************************** */
  if ((!isset($_SERVER['HTTP_USER_AGENT']) || $_SERVER['HTTP_USER_AGENT'] != $_SDATA['userAgent']) &&
      $_VDATA['sp.lock'] == "false" &&
      $_VDATA['sp.cron'] == "false" &&
      $_VDATA['sp.interval'] &&
      $_VDATA['sp.time'] < (time() - $_VDATA['sp.interval'] * 3600)) {
    $rpage = new OS_Fetcher($_VDATA['sp.pathto']);
    $rpage->request = "HEAD";
    $rpage->accept = array("text/html", "application/xhtml+xml", "text/xml");
    $rpage->fetch();

    if (count(preg_grep("/^Orcascript: Search_Spider/", $rpage->headers))) {
      OS_setData("s.spkey", md5(time()));
      $conn2 = pfsockopen($rpage->parsed['realhost'], $rpage->parsed['port'], $erstr, $errno, 5);
      @fwrite($conn2, "GET {$rpage->parsed['path']}?key={$_VDATA['s.spkey']} HTTP/1.0\r\nHost: {$rpage->parsed['hostport']}\r\nUser-Agent: {$_SDATA['userAgent']}\r\nReferer: http://{$_SERVER['HTTP_HOST']}{$_SERVER["REQUEST_URI"]}\r\n\r\n");
    }
  }
}

?>